﻿using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

    #region CellEditButtonElementToolTipCreationFilter Class

    internal class CellEditButtonElementToolTipCreationFilter :
        IToolTipItem,
        IUIElementCreationFilter
    {        
        internal static CellEditButtonElementToolTipCreationFilter VALUE = new CellEditButtonElementToolTipCreationFilter();

        /// <summary>
        /// Invoked when the tooltip information for an element is required.
        /// </summary>
        /// <param name="mousePosition">Current mouse position</param>
        /// <param name="element">Element for which the tooltip will be displayed</param>
        /// <param name="previousToolTipElement">Previous element that the tooltip was displayed for or null.</param>
        /// <param name="toolTipInfoDefault">Default ToolTipInfo structure initialized based on the mouse position. The structure may be modified and returned from the method.</param>
        ToolTipInfo Infragistics.Win.IToolTipItem.GetToolTipInfo(Point mousePosition, UIElement element, UIElement previousToolTipElement, ToolTipInfo toolTipInfoDefault)
        {
            EditButtonUIElement editButtonUIElement = element as EditButtonUIElement;
            if (null != editButtonUIElement)
                toolTipInfoDefault.ToolTipText = this.GetEditButtonToolTipTextResolved(editButtonUIElement);

            // By default the tool tip info mechanism has no auto pop delay. Meaning the
            // tool tip stays visible until the mouse leaves the element. To maintain
            // the previous UltraGrid behavior of auto-popping, set the AutoPopDelay to
            // -1 which means use the default auto pop delay.
            // 
            toolTipInfoDefault.AutoPopDelay = -1;

            return toolTipInfoDefault;
        }

        public void AfterCreateChildElements(UIElement parent)
        {

        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            if (parent is EditButtonUIElement)
            {
                parent.ToolTipItem = CellEditButtonElementToolTipCreationFilter.VALUE;
            }

            return false;
        }

        private string GetEditButtonToolTipTextResolved(EditButtonUIElement editButtonUIElement)
        {
            var cell = editButtonUIElement.GetContext(typeof(UltraGridCell)) as UltraGridCell;
            if (null != cell)
                return GetEditButtonToolTipTextResolved(cell);

            return null;
        }

        private string GetEditButtonToolTipTextResolved(UltraGridCell cell)
        {
            string tooltipText = cell.GetEditButtonToolTipText();
            if (false == string.IsNullOrEmpty(tooltipText))
                return tooltipText;

            tooltipText = cell.Column.GetEditButtonToolTipText();
            if (false == string.IsNullOrEmpty(tooltipText))
                return tooltipText;

            return null;
        }
    }

#endregion // CellEditButtonElementToolTipCreationFilter Class

#region EditButtonToolTipTextPropertyExtensionProvider
internal static class EditButtonToolTipTextPropertyExtensionProvider
{
    internal static Dictionary<UltraGridCell, string> cellTooltipDictionary = new Dictionary<UltraGridCell, string>();
    internal static Dictionary<UltraGridColumn, string> columnTooltipDictionary = new Dictionary<UltraGridColumn, string>();


    public static void SetEditButtonToolTipText(this UltraGridCell cell, string tooltipText)
    {
        cellTooltipDictionary[cell] = tooltipText;
    }

    public static string GetEditButtonToolTipText(this UltraGridCell cell)
    {
        string tooltipText;
        if (cellTooltipDictionary.TryGetValue(cell, out tooltipText))
            return tooltipText;

        return null;
    }

    public static void SetEditButtonToolTipText(this UltraGridColumn column, string tooltipText)
    {
        columnTooltipDictionary[column] = tooltipText;
    }

    public static string GetEditButtonToolTipText(this UltraGridColumn column)
    {
        string tooltipText;
        if (columnTooltipDictionary.TryGetValue(column, out tooltipText))
            return tooltipText;

        return null;

    }
}
#endregion // EditButtonToolTipTextPropertyExtensionProvider